#include "../include/config.hpp"
#include "../include/blender.hpp"
#include "../include/MaterialParser.hpp"

namespace MQO {

static py::object projector;


void MaterialParser::assignProperty(
		py::object mat, int key, MaterialParser::pointer_t const &val )const
{
	key & ( 1 << 4 ) ? assignIntegral( mat, key, val ):
	key & ( 1 << 5 ) ? assignFloat( mat, key, val ):
	key & ( 1 << 6 ) ? assignString( mat, key, val ):
	key & ( 1 << 7 ) ? assignList( mat, key, val ): 0;
}

void MaterialParser::assignIntegral(
		py::object mat, int key, MaterialParser::pointer_t const &val )const
{
	unsigned int &ival = *reinterpret_cast<unsigned int*>( val.get() );

	switch(key){
	case ShadingType:
		if( ival > 2 ){
			mat.attr("setSpecShader")( ival - 2 );
		}
		return;

	case Projection:
		if( ival > 0 ){
			projector = Blender.Object().attr("New")("Empty");
			Blender.Scene().attr("GetCurrent")().attr("objects").attr("link")( projector );

			py::list textures( mat.attr("getTextures")() );

			for( int i = 0; i < 3; ++i ){
				py::object mtex = textures[i];
				if( !mtex ){
					continue;
				}
				mtex.attr("texco") = Blender.Texture().attr("TexCo").attr("OBJECT");
				mtex.attr("object") = projector;
				mtex.attr("mapping") = ( ival == 1 ? 0: ival );
			}
		}
		return;

	case VertexColor:
		;
	}
}

void MaterialParser::assignFloat(
		py::object mat, int key, MaterialParser::pointer_t const &val )const
{
	double &fval = *reinterpret_cast<double*>( val.get() );

	switch(key){
	case Diffuse:
	case Ambient:
	case Emit:
		{
			static char *attrname[3] = {
				"setRef", "setAmb", "setEmit"
			};
			mat.attr( attrname[ key - Diffuse ] )( fval );
			return;
		}
	case Specular:
		{
			mat.attr("setSpec")( fval * 2 );
			return;
		}
	case Hard:
		{
			mat.attr("setHardness")( (unsigned int)( fval * 5.11 ) );
			return;
		}
	}
}

void MaterialParser::assignString(
		py::object mat, int key, MaterialParser::pointer_t const &val )const
{
	std::string &sval = *reinterpret_cast<std::string*>( val.get() );

	switch(key){
	case ColorMap:
	case NormalMap:
	case AlphaMap:
		createTexture( mat, key - ColorMap, sval );
		return;
	}
}

void MaterialParser::assignList(
		py::object mat, int key, MaterialParser::pointer_t const &val )const
{
	if( key == ProjectRot && projector ){
		boost::array<double,3> &aval = *reinterpret_cast<boost::array<double,3>*>( val.get() );
		py::list lval( py::make_tuple( aval[0], aval[2], aval[1] ) );

		py::object angle = Blender.Mathutils().attr("Euler")( lval );
		angle = angle.attr("toMatrix")().attr("resize4x4")();
		projector.attr("setMatrix")(angle);
		return;
	}
	else if( key == RGB ){
		boost::array<double,4> &aval = *reinterpret_cast<boost::array<double,4>*>( val.get() );
		py::list rgb( py::make_tuple( aval[0], aval[1], aval[2] ) );

		mat.attr("setRGBCol")( rgb );
		mat.attr("setAlpha")( aval[3] );

		if (aval[3] != 1)
		{
			py::object ztransp = Blender.Material().attr("Modes").attr("ZTRANSP");
			mat.attr("mode") = mat.attr("mode") | ztransp;
		}

		return;
	}
}

void MaterialParser::createTexture( py::object mat, int key, std::string const &name )const
{
	static const char * mapname[3] = {
		"ColorMap",
		"NormalMap",
		"AlphaMap"
	};
	std::string texname(name);

	if( fs::exists( fs::path(texname, fs::native) ) == false )
	{
		if( Blender.GANGu().has_key( mapname[key] ) == false ){
			std::cout << "Unable to Load " << mapname[key] << "[" << texname << "]\n";
			return;
		}
		std::string mapdir;

		if ( py::extract<int>( Blender.GANGu()["UseTargetDir"] ) ){
			fs::path mqo = py::extract<std::string>( Blender.GANGu()["Target"] );
			mapdir = mqo.branch_path().string();
		}
		else{
			mapdir = py::extract<std::string>( Blender.GANGu()[ mapname[key] ] );
		}

		fs::path target = fs::path(
			mapdir,
			fs::native ) / texname;
		if( fs::exists( target ) == false ){
			std::cout << "Unable to Load " << mapname[key] << "[" << target.string() << "]\n";
			return;
		}

		texname = target.string();
	}
	py::object texture = Blender.Texture().attr("New")( mapname[key] );
	py::object image = Blender.Image().attr("Load")( texname );
	texture.attr("setType")("Image");
	texture.attr("setImage")( image );

	if( key == 2 ){
		texture.attr("setImageFlags")("UseAlpha");
	}

	static const char *cmapto[3] = { "COL", "NOR", "ALPHA" };

	int mapto = py::extract<int>(
		Blender.Texture().attr("MapTo").attr( cmapto[key] ) );

	int texco = py::extract<int>( Blender.Texture().attr("TexCo").attr("UV") );

	mat.attr("setTexture")( key, texture, texco, mapto );
}

}//MQO
